#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------

__author__ = "Zafer Aydin"
__date__ = "01/25/2009"

# FILE: generate_data_yao_tlag_sig_linear.py
# PROJECT: Protein Secondary Structure Prediction

usage= """

USAGE: python genenate_data_yao_tlag_sig_linear.py <dataset_file> <number_of_cv_folds> <data_dir> <blast_dir> <blastdb_dir> <pssm_window_size> <ss_window_size> <randomly_split_cv_folds> <generate_pssm> <sigmoid_transformation_flag> <linear_transformation_flag> <fold_root> <train_files_root> <train_ve_files_root> <test_files_root> <test_postp_files_root> <test_vit_files_root>

DESCRIPTION: 

Generates various files that will be used by the runTrainTest.sh for cross-validation experiment. Files with following extensions are produced: .list, .obs, .fasta, .pssm, .vedat. PSI-BLAST is used to generate PSSM profiles.

The following functions are called during the execution of generate_data.py:

generate_folds_list_file
generate_fold_list_files
give_me_aa_number
give_me_ss_number
generate_obs_file
run_psiblast_for_a_protein
generate_fasta_obs_vedat_files
make_list_files

-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
generate_folds_list_file

Generates a file called <fold_root>s.list that contains the strings <fold_root>i.list with 0 <= i <= <number_of_cv_folds>-1. The filenames are stored in separate lines.

-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
generate_fold_list_files

Starts from the original dataset and randomly divides it into <number_of_cv_folds> folds. It generates the <fold_root>i.list files where 0 <= i <= <number_of_cv_folds>-1. Each <fold_root>i.list contains a list of the .obs filenames (with relative paths) for proteins in the (i+1)^th fold. If the number of proteins in the dataset is not divisible by the number of folds then the remaining M proteins is equally distributed among the last M folds where M is the remainder of the integer division: #dataset proteins / <number_of_cv_folds>.

-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
give_me_aa_number

Returns a number that represents the amino acid type. The amino acids are assigned numbers from 0 to 20. The number 0 is assigned for X and other possible amino acid types (e.g. J, B, Z). Numbers 1 to 20 are assigned to the 20 common amino acids.

-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
give_me_ss_number

Returns a number that represents the secondary structure type of a particular amino acid. The input label can have 8 possible values ('H', 'B', 'E', 'G', 'I', 'T', 'S', ' '). Secondary structure labels are assigned numbers from 1 to 8. 0 is returned when the input label is not one of the 8 labels of secondary structure. Residues in the first segment and those in the last segment will be assigned 9 and 10 respectively by the function called generate_obs_file.  

-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
generate_obs_file

Returns an .obs file for a protein. It puts the amino acid sequence into the first column, and the secondary structure label into the second column. The columns are tab-separated. Amino acids and secondary structure labels are represented by numbers. The secondary structure labels of the residues that belong to the first segment are represented by 9 and those that form the last segment by 10.  

-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
run_psiblast_for_a_protein

Performs a PSI-BLAST search against the NR database of NCBI for a protein that is stored in fasta_filename. The fasta file should be in fasta format, containing two lines: (1) protein's id (e.g. >1bcx) (2) amino acid sequence.

-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
generate_fasta_obs_vedat_files

Generates .fasta, .obs,. and .vedat files for each protein in the dataset. .fasta files will be used to run PSI-BLAST and generate PSSM-profiles. .obs and .vedat files will be used by runTrainTest.sh for training.


-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
make_list_files

Generates .list files that will be used by runTrainTest.sh. The files that are generated contains lists of files that will be used for training and testing the DBN model. The following files are output to the user: <train_files>i.list, <train_ve_files>i.list, <test_files>i.list, <test_ve_files>i.list, <test_postp_files>i.list, <test_vit_files>i.list with 0 <= i <= <number_of_cv_folds>-1. 

"""

def generate_folds_list_file(  number_of_cv_folds, dataset_dir, folds_list_filename, fold_root ):
	
	folds_list_file = open (folds_list_filename, 'w')
	
	template_str = dataset_dir + fold_root

	for i in range(0, number_of_cv_folds, 1):
		
		filename = template_str
		filename = filename + str(i) + '.list' + '\n'
		folds_list_file.write(filename)

	folds_list_file.close()

def generate_fold_list_files( dataset_filename, number_of_cv_folds, dataset_dir, folds_list_filename ):

	dataset_file = open (dataset_filename, 'r')	
	
	number_of_proteins = 0
	
	#Count the number of proteins in the dataset and extract the protein ids into a list
	protein_ids = []
	for line in dataset_file:
		
		if (line[0] == '>'):
			number_of_proteins += 1	
			protein_ids.append( line )
	
	#Determine the number of proteins in each fold					
	quotient = number_of_proteins / number_of_cv_folds
	remainder = number_of_proteins % number_of_cv_folds
	
	number_proteins_in_each_fold = [0] * number_of_cv_folds

	for i in range(0, number_of_cv_folds, 1):
		
		number_proteins_in_each_fold[i] = quotient

	for i in range(0, remainder, 1):

		number_proteins_in_each_fold[-1-i] += 1

	#Randomly divide into N folds	
	protein_indices = range( number_of_proteins )
	random.shuffle( protein_indices )	
	
	folds_list_file = open(folds_list_filename, 'r')

	start_index = 0
	for i in range(0, number_of_cv_folds, 1):
		
		end_index = start_index + number_proteins_in_each_fold[i] 
				
		fold_filename = folds_list_file.readline()
		fold_filename = fold_filename.rstrip()
		fold_list_file = open( fold_filename, 'w')
	
		for j in range(start_index, end_index, 1):

			a_protein_id = protein_ids[protein_indices[j]]
			subfilename = a_protein_id[1:].rstrip()
			subfilename2 = subfilename.replace('.all', '')

			fold_list_file.write(dataset_dir + subfilename2 + '\n')

		fold_list_file.close()
		start_index = end_index
	
	folds_list_file.close()
	dataset_file.close()	

def give_me_aa_number( aa ): 
 
        aa_labels = 'XARNDCQEGHILKMFPSTWYV'
        aa_number = aa_labels.find(aa)
 
        if ( aa_number == -1 ):
 
                return 0 
        else:  
                return aa_number
 
def give_me_ss_number( ss ): 
 
        ss_labels = 'HBEGITS '
        ss_number = ss_labels.find(ss)
 
        if ( ss_number == -1 ):
                 
                return 0 
        else: 
                return ss_number+1

def give_me_ss_number_3_state( ss ):

	if (ss == 'H'):
		
		return 1
	
	elif (ss == 'E'):

		return 2

	elif (ss == 'L'):

		return 3

	else: 
		return 0
	
def reverse_pssm( pssm ):

	n_rows = len( pssm )
	n_iterations = n_rows / 2
	
	for i in range(0, n_iterations, 1):

		temp = pssm[i]
		pssm[i] = pssm[n_rows-1-i]
		pssm[n_rows-1-i] = temp

	return pssm
	
def generate_obs_file( dataset_dir, pssm_filename, obs_filename, pssm_window_size, ss_window_size, ss_seq, reverse_model_flag, sigmoid_transformation, linear_transformation, Dmax):

        obs_file = open( obs_filename, 'w')
	pssm_file = open( pssm_filename, 'r')

	pssm = []

	#Skip the first three lines of the PSSM file    
        line = pssm_file.readline()
        line = pssm_file.readline()
        line = pssm_file.readline()
	
 	done = 0
	number_of_aas = 0
	number_of_columns = 20

        while not done:

                line = pssm_file.readline()

                if ( len(line) < 10 ):
                        
                        done = 1
                        continue

		aa_index = int(line[:5])
                aa_type = line[6:7]

		start_index = 10
		pssm_row = []

		for i in range(0, number_of_columns, 1):

			end_index = start_index+1
                        done2 = 0
                        while not done2:
                        
                                if (line[end_index:end_index+1] == ' '):
                                        done2 = 1
                                        continue

                                else:
                                        end_index += 1
	
                        pssm_value = int(line[start_index:end_index])
			#a_random_value = random.normalvariate(0,0.05)
			#pssm_value += a_random_value

			if (sigmoid_transformation == 1):
                        	
				transformed_pssm = 1 / (1 + exp(-1*pssm_value))

			elif (linear_transformation == 1):

				if (pssm_value > 7):
				
					transformed_pssm = 1.0

				elif (pssm_value < -7):

					transformed_pssm = 0.0
				
				else:

					transformed_pssm = pssm_value / 14.0 + 0.5

                        #if ( transformed_pssm < 1e-43 ):
			#	log_transformed_pssm = -99.00
                               
                        #else:
                        #       log_transformed_pssm = log(transformed_pssm)
                                
			pssm_row += [ transformed_pssm ]
			#pssm_row += [ pssm_value ]

			start_index = end_index+1

		pssm.extend( [pssm_row] )
 		number_of_aas += 1
	
	if ( reverse_model_flag == 0 ):

		pssm = pssm

	elif ( reverse_model_flag == 1 ):

		pssm = reverse_pssm( pssm )

	number_of_loops = 0
	for i in range(number_of_aas):

		if (ss_seq[i] == 'L'):

			number_of_loops += 1

	#generate random positions for the first <pssm_window_size> artificially generated profile vectors
	for i in range(0, pssm_window_size, 1):

		a_random_int = random.randint(1, number_of_loops)

		loop_counter = 0		
		for j in range(number_of_aas):

			if (ss_seq[j] == 'L'):
				loop_counter += 1
			
			if (loop_counter == a_random_int):
				random_pssm_index = j
				break

		pssm_curr = pssm[random_pssm_index]  #Since both pssm and ss_seq are reversed in the generate_fasta_obs_files function 

		for j in range(0, 20, 1):
       
	              	obs_file.write('%.7f ' % pssm_curr[j] )

		obs_file.write('%d ' % 2)
		obs_file.write('%d ' % 5)
		obs_file.write('%d' % 0)
		obs_file.write('\n')

	Di_seq = [0]*number_of_aas
	Fi_seq = [0]*number_of_aas	
	ss_label_pre = ss_seq[0]

	#print len(ss_seq)
	#print number_of_aas

	curr_segment_length = 0
	segment_start_index = 0
	segment_end_index = number_of_aas-1

	for i in range(1, number_of_aas, 1):

		ss_label_curr = ss_seq[i]
		ss_label_pre = ss_seq[i-1]

		if (i == number_of_aas - 1):

                	Fi_seq[i] = 1

		if ((ss_label_curr != ss_label_pre) or ((ss_label_curr == ss_label_pre) and (i == number_of_aas - 1))):
		
			if (ss_label_curr != ss_label_pre):
			
				Fi_seq[i-1] = 1	
				curr_segment_length = i - segment_start_index 
				segment_end_index = i-1
			
			elif ((ss_label_curr == ss_label_pre) and (i == number_of_aas - 1)):

				curr_segment_length = i - segment_start_index + 1
                                segment_end_index = i

			for j in range(segment_start_index, segment_end_index+1, 1):

				within_index = j-segment_start_index

				if (within_index < curr_segment_length - Dmax):
				
					Di = Dmax

				else: 
					Di = curr_segment_length-within_index

				Di_seq[j] = Di-1			
			
			segment_start_index = segment_end_index + 1

	for i in range(0, number_of_aas, 1):

		pssm_curr = pssm[i]

		for j in range(0, 20, 1):
	
			obs_file.write('%.7f ' % pssm_curr[j] )

		ss_label = give_me_ss_number_3_state(ss_seq[i])
	        ss_label -= 1	
		obs_file.write('%d ' % ss_label )
		obs_file.write('%d ' % Di_seq[i] )
		obs_file.write('%d ' % Fi_seq[i] )
		
		if ((Di_seq[i] == 0) and (Fi_seq[i] == 0)):

			print 'HEY Di=0 Fi=0 case i=%d %s' % (i, obs_filename)
	
		obs_file.write('\n')
		
	pssm_file.close()
	obs_file.close()

def run_psiblast_for_a_protein( fasta_filename, pssm_filename, blast_dir, blastdb_dir ):

	blast_command = blast_dir + 'blastpgp'
	blastdb_arg = blastdb_dir + 'nr'
	os.system("%s -i %s -o alignments -Q %s -j 3 -e 0.001 -h 1e-10 -d %s" % (blast_command, fasta_filename, pssm_filename, blastdb_arg))

def reduce_from_8_to_3_state_EHL_mapping( ss_seq ):

	str_length = len(ss_seq)
	
	ss_seq_red = ''

	for i in range(0, str_length, 1):

		if ( (ss_seq[i] == 'H') or (ss_seq[i] == 'G') or (ss_seq[i] == 'I') ):
			
			ss_seq_red += 'H'

		elif ( (ss_seq[i] == 'E') or (ss_seq[i] == 'B') or (ss_seq[i] == 'e') or (ss_seq[i] == 'b')):
			
			ss_seq_red += 'E'

		elif ( (ss_seq[i] == 'S') or (ss_seq[i] == 'T') or (ss_seq[i] == ' ') or (ss_seq[i] == 'L')):
			ss_seq_red += 'L'

	return ss_seq_red

def generate_fasta_obs_files( dataset_filename, dataset_dir, blast_dir, blastdb_dir, pssm_files_dir, pssm_window_size, ss_window_size, reverse_model_root, reverse_model_flag, generate_pssm, sigmoid_transformation, linear_transformation, Dmax ):

        dataset_file = open( dataset_filename, 'r')
        fastas_list_filename = dataset_dir + 'fastas.list'
        fastas_list_file = open( fastas_list_filename, 'w')

        done = 0

	print 'Generating .fasta files'
        print '...'
	
	n_seqs = 0

        while not done:

                id = dataset_file.readline()

                if ( len(id) < 1 ):

                        done = 1
                        continue

                aa_seq = dataset_file.readline()
                ss_seq = dataset_file.readline()

                if ( len(aa_seq) != len(ss_seq) ):

                        print 'The amino acid sequence and secondary structure string are not the same length for protein %s' % id

                fasta_id = id.replace('.all', '')

                if ( id[0] == '>' ):

                        subfilename = id[1:].rstrip()
                else:
                        subfilename = id.rstrip()

                subfilename2 = subfilename.replace('.all', '')
		fasta_filename_seed = subfilename2 + '.fasta'
                fastas_list_file.write(fasta_filename_seed + '\n')
                fasta_filename = dataset_dir + fasta_filename_seed

                fasta_file = open( fasta_filename, 'w')
                fasta_file.write(fasta_id)
                fasta_file.write(aa_seq)
                fasta_file.close()

		n_seqs += 1

        dataset_file.close()
        fastas_list_file.close()

 	print 'Done!\n'

        print 'Generating PSIBLAST profiles (PSSMs)'
        print '...'

        fastas_list_file = open( fastas_list_filename, 'r')

        for line in fastas_list_file:

                fasta_filename_seed = line.rstrip()
                pssm_filename = fasta_filename_seed
                subfilename = pssm_filename.replace('.fasta', '')
		fasta_filename = dataset_dir + fasta_filename_seed

		pssm_filename = dataset_dir + subfilename + '.pssm'

		if ( reverse_model_flag == 0 ):
			
			if (generate_pssm == 1):	
				
				print 'Running PSIBLAST for %s ' % fasta_filename_seed
		        	run_psiblast_for_a_protein( fasta_filename, pssm_filename, blast_dir, blastdb_dir )

        fastas_list_file.close()
        
	print 'Done!\n'

        print 'Generating .obs files'
        print '...'

	done = 0
	dataset_file = open( dataset_filename, 'r')

	file_counter = 0

	while not done:

                id = dataset_file.readline()

                if ( len(id) < 1 ):

                        done = 1
                        continue

		aa_seq = dataset_file.readline()
                ss_seq = dataset_file.readline()
		aa_seq = aa_seq.rstrip()
		naa = len(aa_seq)
		ss_seq = ss_seq[0:naa]
                ss_seq_red = reduce_from_8_to_3_state_EHL_mapping( ss_seq )

		if ( reverse_model_flag == 0 ):

                	aa_seq = aa_seq
	                ss_seq = ss_seq
			ss_seq_red = ss_seq_red

		elif ( reverse_model_flag == 1 ):

			aa_seq = aa_seq[::-1]
			ss_seq = ss_seq[::-1]
			ss_seq_red = ss_seq_red[::-1]

		if ( id[0] == '>' ):

                        subfilename = id[1:].rstrip()
                else:
                        subfilename = id.rstrip()

                subfilename2 = subfilename.replace('.all', '')
                
		if ( reverse_model_flag == 0 ):

			#pssm_filename = dataset_dir + subfilename2 + '.pssm'
			#pssm_filename = '/net/noble/vol2/home/zafer/projects/pssp/gmtk/zafers_models/time_lag_style/pssms/' + subfilename2 + '.pssm'
                        pssm_filename = pssm_files_dir + subfilename2 + '.pssm'
			obs_filename = dataset_dir + subfilename2 + '.obs'
	                generate_obs_file( dataset_dir, pssm_filename, obs_filename, pssm_window_size, ss_window_size, ss_seq_red, reverse_model_flag, sigmoid_transformation, linear_transformation, Dmax )

                        if (file_counter == 0):
                                obs_filename_2 = dataset_dir + '1.obs'
                                os.system("\cp %s %s" % (obs_filename, obs_filename_2))

                elif ( reverse_model_flag == 1 ):

			#pssm_filename = dataset_dir + subfilename2 + '.pssm'
			#pssm_filename = '/net/noble/vol2/home/zafer/projects/pssp/gmtk/zafers_models/time_lag_style/pssms/' + subfilename2 + '.pssm'
			pssm_filename = pssm_files_dir + subfilename2 + '.pssm'
			obs_filename = dataset_dir + subfilename2 + reverse_model_root + '.obs'
			generate_obs_file( dataset_dir, pssm_filename, obs_filename, pssm_window_size, ss_window_size, ss_seq_red, reverse_model_flag, sigmoid_transformation, linear_transformation, Dmax )

		file_counter += 1
	
	dataset_file.close()
        print 'Done!\n'

def make_list_files( folds_list_filename, dataset_dir, train_files_root, test_files_root, test_postp_files_root, w_test_files_root, w_test_postp_files_root, w_train_files_root, reverse_model_root, reverse_model_flag, sigmoid_transformation, linear_transformation ):

        folds_list_file = open( folds_list_filename, 'r')
        folds_list_file_inside = folds_list_file.readlines()
        n_folds = len(folds_list_file_inside)
        fold_indices = range(0, n_folds)

	for j in range(0, n_folds, 1):

                fold_list_filename = folds_list_file_inside[j].rstrip()
                fold_list_file = open( fold_list_filename, 'r')
                fold_list_file_inside = fold_list_file.readlines()

		if ( reverse_model_flag == 0 ):

	        	test_data_filename = dataset_dir + test_files_root + str(j) + '.list'

		elif ( reverse_model_flag == 1 ):
		
			test_data_filename = dataset_dir + test_files_root + reverse_model_root + str(j) + '.list'

                #Prepare the test_file for the j^th fold
		test_data_file = open( test_data_filename, 'w')

		for i in range(0, len(fold_list_file_inside), 1):
		
			a_row = str(fold_list_file_inside[i])
			a_row = a_row.rstrip()
	
			if ( reverse_model_flag == 0 ):

				test_data_file.write( a_row + '.obs\n' )

			elif ( reverse_model_flag == 1 ):

				test_data_file.write( a_row + reverse_model_root + '.obs\n' )
							
                test_data_file.close()
                fold_list_file.close()

                #Prepare the postp file for the j^th fold
		fold_list_file = open( fold_list_filename, 'r')
                fold_list_file_inside2 = fold_list_file.readlines()

                list_length = len(fold_list_file_inside2)

                if ( reverse_model_flag == 0 ):

			test_postp_data_filename = dataset_dir + test_postp_files_root + str(j) + '.list'

		elif ( reverse_model_flag == 1 ):

			test_postp_data_filename = dataset_dir + test_postp_files_root + reverse_model_root + str(j) + '.list'

                test_postp_data_file = open( test_postp_data_filename, 'w')

                for i in range(0, list_length, 1):

                        an_obs_file = fold_list_file_inside2[i]

			if ( reverse_model_flag == 0 ):

				if ((sigmoid_transformation == 1) and (linear_transformation == 0)):
	
		                        a_test_postp_file = an_obs_file.replace('\n', '.test.postp.sigmoid\n')
			
				elif ((sigmoid_transformation == 0) and (linear_transformation == 1)):
	
					a_test_postp_file = an_obs_file.replace('\n', '.test.postp.linear\n')
						
			elif ( reverse_model_flag == 1 ):

				if ((sigmoid_transformation == 1) and (linear_transformation == 0)):
	
					a_test_postp_file = an_obs_file.replace('\n', reverse_model_root + '.test.postp.sigmoid\n')

				elif ((sigmoid_transformation == 0) and (linear_transformation == 1)):

					a_test_postp_file = an_obs_file.replace('\n', reverse_model_root + '.test.postp.linear\n')

                        a_test_postp_filename = a_test_postp_file.replace(dataset_dir, '')
			test_postp_data_file.write(a_test_postp_filename)

                fold_list_file.close()
                test_postp_data_file.close()
              
		#Prepare w-test w_test_postp and training files
                remaining_folds = fold_indices[0:j]+fold_indices[j+1:n_folds]

		for k in range(0, n_folds-1, 1):

			index_w_test = remaining_folds[k]
			w_test_fold_list_filename = folds_list_file_inside[index_w_test].rstrip()
                        w_test_fold_list_file = open( w_test_fold_list_filename, 'r')
                        w_test_fold_list_file_inside = w_test_fold_list_file.readlines()

			#Prepare the w-test file for the (j,k)^th fold
                        if ( reverse_model_flag == 0 ):

				w_test_data_filename = dataset_dir + w_test_files_root + str(j) + str(k) + '.list'

                        elif ( reverse_model_flag == 1 ):

				w_test_data_filename = dataset_dir + w_test_files_root + reverse_model_root + str(j) + str(k) + '.list'

			w_test_data_file = open( w_test_data_filename, 'w')
	
			for i in range(0, len(w_test_fold_list_file_inside), 1):

				a_row = str(w_test_fold_list_file_inside[i]).rstrip()					

				if ( reverse_model_flag == 0 ):

                                        w_test_data_file.write( a_row + '.obs\n' )

                                elif ( reverse_model_flag == 1 ):

                                        w_test_data_file.write( a_row + reverse_model_root + '.obs\n' )
					
			w_test_data_file.close()
                        w_test_fold_list_file.close()

			#Prepare the w_test_postp file for the (j,k)^th fold
                        if ( reverse_model_flag == 0 ):

                                w_test_postp_data_filename = dataset_dir + w_test_postp_files_root + str(j) + str(k) + '.list'

                        elif ( reverse_model_flag == 1 ):

                                w_test_postp_data_filename = dataset_dir + w_test_postp_files_root + reverse_model_root + str(j) + str(k) + '.list'

			w_test_postp_data_file = open( w_test_postp_data_filename, 'w')

                        for i in range(0, len(w_test_fold_list_file_inside), 1):

                                a_row = w_test_fold_list_file_inside[i]

                                if ( reverse_model_flag == 0 ):

					if ((sigmoid_transformation == 1) and (linear_transformation == 0)):

						a_test_postp_filename = a_row.replace('\n', '.w_test.postp.sigmoid\n')

                                	elif ((sigmoid_transformation == 0) and (linear_transformation == 1)):

                                        	a_test_postp_filename = a_row.replace('\n', '.w_test.postp.linear\n')

                                elif ( reverse_model_flag == 1 ):

					if ((sigmoid_transformation == 1) and (linear_transformation == 0)):

	                                        a_test_postp_filename = a_row.replace('\n', reverse_model_root + '.w_test.postp.sigmoid\n')

					elif ((sigmoid_transformation == 0) and (linear_transformation == 1)):

						a_test_postp_filename = a_row.replace('\n', reverse_model_root + '.w_test.postp.linear\n')

                                a_test_postp_filename = a_test_postp_filename.replace(dataset_dir, '')
                                w_test_postp_data_file.write(a_test_postp_filename)

                        w_test_postp_data_file.close()

			#Prepare the training-data file for the (j,k)^th fold
                        remaining_folds2 = remaining_folds[0:k] + remaining_folds[k+1:n_folds-1]
                        n_remaining_folds2 = len(remaining_folds2)

                        if ( reverse_model_flag == 0 ):

                                train_data_filename = dataset_dir + w_train_files_root + str(j) + str(k) + '.list'

                        elif ( reverse_model_flag == 1 ):

                                train_data_filename = dataset_dir + w_train_files_root + reverse_model_root + str(j) + str(k) + '.list'

                        train_data_file = open( train_data_filename, 'w')

                        for m in range(0, n_remaining_folds2, 1):

                                fold_list_filename = folds_list_file_inside[remaining_folds2[m]].rstrip()
                                fold_list_file = open( fold_list_filename, 'r')
                                fold_list_file_inside = fold_list_file.readlines()

                                for i in range(0, len(fold_list_file_inside), 1):

                                        a_row = str(fold_list_file_inside[i])
                                        a_row = a_row.rstrip()

                                        if ( reverse_model_flag == 0 ):

                                                train_data_file.write( a_row + '.obs\n' )

                                        elif ( reverse_model_flag == 1 ):

                                                train_data_file.write( a_row + reverse_model_root + '.obs\n' )

                                fold_list_file.close()

                        train_data_file.close()
	
        folds_list_file.close()

        number_of_folds = n_folds
        folds_list_file = open( folds_list_filename, 'r')

        folds_list_file_inside = folds_list_file.readlines()

        for i in range(0, number_of_folds, 1):

		if ( reverse_model_flag == 0 ):
	
	                train_data_filename = dataset_dir + train_files_root + str(i) + '.list'

		elif ( reverse_model_flag == 1 ):

			train_data_filename = dataset_dir + train_files_root + reverse_model_root + str(i) + '.list'

                train_data_file = open( train_data_filename, 'w')

                for j in range(0, number_of_folds, 1):

                        if (j == i):
                                continue

                        fold_list_filename = folds_list_file_inside[j].rstrip()
			#print fold_list_filename

                        fold_list_file = open( fold_list_filename, 'r')
                        fold_list_file_inside = fold_list_file.readlines()

			for k in range(0, len(fold_list_file_inside), 1):

                        	a_row = str(fold_list_file_inside[k])
	                        a_row = a_row.rstrip()

                        	if ( reverse_model_flag == 0 ):

                                	train_data_file.write( a_row + '.obs\n' )

	                        elif ( reverse_model_flag == 1 ):

        	                        train_data_file.write( a_row + reverse_model_root + '.obs\n' )

                        fold_list_file.close()

                train_data_file.close()

        folds_list_file.close()

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
# This is the main program 

import sys
import random
import os
from math import *

if __name__ == "__main__":

	if ( len(sys.argv) != 22):

		sys.stderr.write(usage)
		sys.exit(-1)
	
	dataset_filename = sys.argv[1]
	number_of_cv_folds = int(sys.argv[2])
	dataset_dir = sys.argv[3]
	blast_dir = sys.argv[4]
	blastdb_dir = sys.argv[5]
	pssm_files_dir = sys.argv[6]
	pssm_window_size = int(sys.argv[7])
	ss_window_size = int(sys.argv[8])
	fold_root = sys.argv[9]
	randomly_split_cv_folds = int(sys.argv[10])
	generate_pssm = int(sys.argv[11])
	sigmoid_transformation = int(sys.argv[12])
	linear_transformation = int(sys.argv[13])
	train_files_root = sys.argv[14]
	test_files_root = sys.argv[15]
	test_postp_files_root = sys.argv[16]
	w_test_files_root = sys.argv[17]
        w_test_postp_files_root = sys.argv[18]
	w_train_files_root = sys.argv[19]
	reverse_model_root = sys.argv[20]
	Dmax = int(sys.argv[21])

	folds_list_filename = dataset_dir + fold_root + 's' + '.list'

	print ""	
	print 'Generating ' + fold_root + 's.list file'
	print '...'
	generate_folds_list_file( number_of_cv_folds, dataset_dir, folds_list_filename, fold_root )
	print 'Done!\n'

	if (randomly_split_cv_folds == 1):
 	
		print ""
        	print 'Generating ' + fold_root + '.list files'
        	print '...'
        	generate_fold_list_files( dataset_filename, number_of_cv_folds, dataset_dir, folds_list_filename )
        	print 'Done!\n'

	generate_fasta_obs_files( dataset_filename, dataset_dir, blast_dir, blastdb_dir, pssm_files_dir, pssm_window_size, ss_window_size, reverse_model_root, 0, generate_pssm, sigmoid_transformation, linear_transformation, Dmax) #for the NC model
	generate_fasta_obs_files( dataset_filename, dataset_dir, blast_dir, blastdb_dir, pssm_files_dir, pssm_window_size, ss_window_size, reverse_model_root, 1, generate_pssm, sigmoid_transformation, linear_transformation, Dmax) #for the CN model

	print 'Generating other training and testing data files'
	print '...'
	make_list_files( folds_list_filename, dataset_dir, train_files_root, test_files_root, test_postp_files_root, w_test_files_root, w_test_postp_files_root, w_train_files_root, reverse_model_root, 0, sigmoid_transformation, linear_transformation ) #for the NC model
	print 'Done!\n'	
	
	print 'Generating other training and testing data files for the reverse model'
        print '...'
        make_list_files( folds_list_filename, dataset_dir, train_files_root, test_files_root, test_postp_files_root, w_test_files_root, w_test_postp_files_root, w_train_files_root, reverse_model_root, 1, sigmoid_transformation, linear_transformation ) #for the CN model
        print 'Done!\n'


#------------------------------------------------------------------------------


